from .__main__ import Strategy
from array import array
from .computation import dif, ema
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np


class Aroon_settings:
    aroon_up_i = 9
    aroon_down_i = 8

    
class Aroon(Strategy):
    def __init__(self, signal_price, long_asset_prices, if_plot, aroon_settings:Aroon_settings()) -> None:
        '''
        Arron
            - 
        
        signal_price_df: 
            - dataframe 
            

        
        '''
        super().__init__('arron')
        self.aroon_settings = aroon_settings
        self.if_plot = if_plot
        self.signal_price = signal_price
        self.long_asset_prices = long_asset_prices
    
    
    def create_signal(self):
        df = pd.DataFrame({'Close':self.signal_price})
        i = 0
        df['aroon_up'] = df.apply(lambda row: self.arron_up(self.aroon_settings.aroon_up_i, df), axis=1)
        i = 0
        df['aroon_down'] = df.apply(lambda row: self.arron_down(self.aroon_settings.aroon_down_i, df), axis=1)
        
        
        
        
    
    def aroon_up(self, aroon_i, df):
        global i
        i = i + 1 
        if ( i > aroon_i ):
            t = i - df['Close'][i-aroon_i:i].idxmax()
            return (t/aroon_i)*100
        else:
            return 0


    def aroon_down(self, aroon_i, df):
            global i
            i = i + 1
            if(i>aroon_i):
                t = i - df['Close'][i-aroon_i:i].idxmin()
                return (t/aroon_i)*100
            else:
                return 0



import numpy as np; import pandas as pd

def ema(S,N):             #指数移动平均,为了精度 S>4*N  EMA至少需要120周期     alpha=2/(span+1)    
    return pd.Series(S).ewm(span=N, adjust=False).mean().values   

def dif(data):
    later_1day = data[1:]
    before_1day = data[:-1]
    return later_1day - before_1day


from turtle import shape
from .momentum import Momentum
from .double_momentum import DoubleMomentum
import yfinance as yf
import numpy as np

from alpaca_trade_api.rest import REST, TimeFrame
from alpaca_trade_api.stream import Stream
import alpaca_trade_api as tradeapi

from datetime import datetime, timedelta

from tqdm import tqdm
from pytz import timezone
from typing import Tuple

from TradingBot.utils import get_data
import pandas as pd


class APISettings:
    API_KEY_ID = "your"
    SECRET_KEY = "your"
    ENDPOINT = 'https://paper-api.alpaca.markets'

    

class RollingBackTest():
    # '''
    # WARINING:
    # for 1m: due to yahoo finance data restriction, must be within the last 30 days
    # - 
    # '''
    def __init__(self, momentum_settings, api_setting:APISettings(), test_strategy:str, start_testing_date, testing_length, 
                 signal_ticker, long_ticker, short_ticker,
                 data_interval, gap, verbose, skip_morning):
        
        self.start_testing_date = start_testing_date
        self.skip_morning = skip_morning
        self.testing_length = testing_length
        self.gap = gap
        self.test_strategy = test_strategy
        self.signal_ticker = signal_ticker
        self.long_ticker = long_ticker
        self.short_ticker = short_ticker
        self.data_interval = data_interval
        self.verbose =  verbose
        self. momentum_settings =  momentum_settings
        self.api = REST(api_setting.API_KEY_ID, api_setting.SECRET_KEY, api_setting.ENDPOINT)
        
        self.total_rets, self.long_rets, self.short_rets, self.rets_noncomp,  start_dates, end_dates = self.testing()
        self.results = pd.DataFrame({'start_date':start_dates, 'end_date':end_dates, 'cum_ret':self.total_rets,'long_cum_ret':self.long_rets, 'short_cum_ret':self.short_rets,
                                'non_cum_ret': self.rets_noncomp})

            
        
    def testing(self):
        start_dates, end_dates = self.create_rolling_dates()
        print("start testing...")
        test_returns = []
        long_returns = []
        short_returns = []
        rets_noncomp = []
        total_sucs = []
        for start_date, end_date in tqdm(zip(start_dates, end_dates), total=len(end_dates)):
            backtest = BackTest(self.test_strategy, self.momentum_settings, start_date, end_date, self.signal_ticker, self.long_ticker, self.short_ticker, self.data_interval, verbose=self.verbose)
            test_returns.append(backtest.return_total)
            long_returns.append(backtest.return_long)
            short_returns.append(backtest.return_short)
            rets_noncomp.append(backtest.total_ret_noncomp)
            total_sucs.append(backtest.total_suc)
            # try:
            #     backtest = BackTest(self.test_strategy, self.momentum_settings, start_date, end_date, self.signal_ticker, self.long_ticker, self.short_ticker, self.data_interval, verbose=self.verbose, skip_morning=self.skip_morning)
            #     test_returns.append(backtest.return_total)
            #     long_returns.append(backtest.return_long)
            #     short_returns.append(backtest.return_short)
            #     rets_noncomp.append(backtest.total_ret_noncomp)
            #     total_sucs.append(backtest.total_suc)
                
            # except:
            #     test_returns.append(1)
            #     long_returns.append(1)
            #     short_returns.append(1)  
            #     rets_noncomp.append(1)
            #     total_sucs.append(0.5)
        
        return  test_returns, long_returns, short_returns, rets_noncomp,  start_dates, end_dates
        
    
    
    def create_rolling_dates(self):
        start_testing = self.start_testing_date

        nyc = timezone('America/New_York')
        #now = datetime.today().astimezone(nyc)
        days_before = datetime.today() - timedelta(self.testing_length)
        days_before_str = days_before.astimezone(nyc).strftime('%Y-%m-%d')
        calendar = self.api.get_calendar(start=start_testing, end=days_before_str) # get the next closet trading day 
        calender_date_str = [date.date for date in calendar] # .strftime('%Y-%m-%d')
        calender_date_str = calender_date_str[0::self.gap]
        starts = []
        ends = []
        for date in calender_date_str:
            starts.append(date.strftime('%Y-%m-%d'))
            end = date + timedelta(days=self.testing_length)
            ends.append(end.strftime('%Y-%m-%d'))
        
        return (starts, ends)
    
    
    
    
    

class BackTest:
    def __init__(self, test_strategy, momentum_settings, start_date, end_date, signal_ticker, long_ticker, short_ticker, data_inverval, which='Close', verbose=True, skip_morning=False):
        '''
        date format like '2022-05-04'
        
        WARNING:
        match dfs took a lot of time.
        
        '''
        # get data
        nasdaq = get_data(tiker=signal_ticker, start=start_date, end=end_date, which=which, interval = data_inverval)
        tqqq = get_data(tiker=long_ticker, start=start_date, end=end_date, which=which,interval = data_inverval)
        sqqq =  get_data(tiker=short_ticker, start=start_date, end=end_date, which=which, interval = data_inverval)
        # remove the last row, since the last row sometimes contains the latest datapoint (Bug of yfinance)
        nasdaq = nasdaq[:-1]
        tqqq = tqqq[:-1]
        sqqq = sqqq[:-1]
        #print(start_date, end_date, nasdaq.shape[0], tqqq.shape[0], sqqq.shape[0])
        
        # add str date (min-level)
        nasdaq = self.convert_time(nasdaq)
        tqqq = self.convert_time(tqqq)
        sqqq = self.convert_time(sqqq)
        
        # if daily or hourly, you can comment this line to save time.
        nasdaq, tqqq, sqqq = self.match_dfs(nasdaq, tqqq, sqqq)
        
        #print(start_date, end_date, nasdaq.shape[0], tqqq.shape[0], sqqq.shape[0])
        
        assert  nasdaq.shape[0] == tqqq.shape[0] == sqqq.shape[0]
        
        
        self.signal_df = self.convertdata_yfinance(nasdaq)
        self.long_df = self.convertdata_yfinance(tqqq)
        self.short_df = self.convertdata_yfinance(sqqq)
        
        # adding log ret to asset df
        # WARNING: ret on 15:45 = given a strategy on time point 15:45, return you can get (during 15:45 ~ 16:00)
        self.long_df['log_ret'] = np.log(self.long_df['price'].shift(-1)) - np.log(self.long_df['price'])
        self.short_df['log_ret'] = np.log(self.short_df['price'].shift(-1)) - np.log(self.short_df['price'])
        # replace nan to 0
        self.long_df['log_ret'].replace(np.nan, 0, inplace=True)
        self.short_df['log_ret'].replace(np.nan, 0, inplace=True)
        assert self.long_df.shape[0] == self.short_df.shape[0] == self.signal_df.shape[0]
        
        
        # get signal
        if test_strategy == 'momentum':
            self.momentum = Momentum(self.signal_df, momentum_settings, verbose, skip_morning)
        
        # converting signal to vector 
        self.signal_df = self.momentum.signal_df # update
        self.signal_df['long_sig'] = np.squeeze(np.array([self.signal_df['signal'] == 'long']))
        self.signal_df['short_sig'] = np.squeeze(np.array([self.signal_df['signal'] == 'short']))
        
        
        
        # print return 
        self.return_total, self.return_long, self.return_short, self.totals, self.longs, self.shorts = self.compute_comp_return()
        self.total_ret_noncomp, self.long_ret_noncomp, self.short_ret_noncomp = self.compute_noncomp_return()
        self.total_suc, self.long_suc, self.short_suc = self.success_rate()


    def match_dfs(self, df1, df2, df3):
        dfs = [df1, df2, df3]
        all_dates = list(set(list(df1['str_time'].values)))+  list(set(list(df2['str_time'].values))) + list(set(list(df3['str_time'].values)))
        valid_dates = [date for date in all_dates if all_dates.count(date)==3]
        valid_dates = set(valid_dates)
        valid_dates = list(valid_dates)
        
        new_dfs = []
        for df in dfs:
            new_dfs.append(df[df['str_time'].isin(valid_dates)])
        

        return new_dfs[0].sort_values('Date').reset_index(drop=True), new_dfs[1].sort_values('Date').reset_index(drop=True), new_dfs[2].sort_values('Date').reset_index(drop=True)

        
    
    def convertdata_yfinance(self, yfi_data):
        '''
        use open during eval
        
        '''
        def convert_time(row):
            nyc = timezone('America/New_York')
            return row['Date'].to_pydatetime().astimezone(nyc)
        
        yfi_data['time'] = yfi_data.apply(lambda row: convert_time(row), axis=1)
        yfi_data.rename(columns={'Open':'price'}, inplace=True)
        return yfi_data[['time', 'price']]
    

    def convert_time(self, yfi_data):
        '''
        
        
        '''
        def convert_(row):
            return row['Date'].to_pydatetime().strftime('%Y-%m-%d-%H-%M') 
        yfi_data['str_time'] = yfi_data.apply(lambda row: convert_(row), axis=1)
        return yfi_data
        
    
    
    def compute_comp_return(self):
        return_totals = []
        return_longs = []
        return_shorts = []
        for i in range(self.long_df.shape[0]): #len(self.tqqq['log_ret'].values[2:])):
            return_long = np.array(self.long_df['log_ret'].values)[:i+1].dot(np.array(self.signal_df['long_sig'])[:i+1])
            return_longs.append(return_long)
            return_short = np.array(self.short_df['log_ret'].values)[:i+1].dot(np.array(self.signal_df['short_sig'])[:i+1])
            return_shorts.append(return_short)
            return_total = np.exp(return_long + return_short)
            return_totals.append(return_total)
        
        return_long = self.long_df['log_ret'].values.dot(self.signal_df['long_sig'].values)
        return_short = self.short_df['log_ret'].values.dot(self.signal_df['short_sig'].values)
        return_total = return_long + return_short
        
        
        return np.exp(return_total), np.exp(return_long), np.exp(return_short) , return_totals, return_longs, return_shorts
    
    def compute_noncomp_return(self):
        
        return_long = np.mean(np.exp(np.multiply(np.array(self.long_df['log_ret'].values), np.array(self.signal_df['long_sig']))))
        return_short = np.mean(np.exp(np.multiply(np.array(self.short_df['log_ret'].values), np.array(self.signal_df['short_sig']))))
        return_total = (return_long + return_short) / 2
        return return_total, return_long, return_short
        
        
        
    def success_rate(self):
        long_arr = np.array(self.long_df['log_ret'].values)
        short_arr = np.array(self.short_df['log_ret'].values)
        try:
            rate_long = (np.array(long_arr > 0).astype(int).dot(self.signal_df['long_sig'].values.astype(int))) / sum(self.signal_df['long_sig'].values)
        except:
            rate_long = 1
        try:
            rate_short = (np.array(short_arr > 0).astype(int).dot(self.signal_df['short_sig'].values.astype(int))) / sum(self.signal_df['short_sig'].values)
        except:
            rate_short = 1
        try:
            total_rate = ((np.array(long_arr > 0).astype(int).dot(np.array(self.signal_df['long_sig']).astype(int))) + (np.array(short_arr > 0).astype(int).dot(np.array(self.signal_df['short_sig']).astype(int))) ) / (sum(self.signal_df['long_sig'])+sum(self.signal_df['short_sig']))
        except:
            total_rate = 1
        return total_rate, rate_long, rate_short
        
    # def get_data(self, tiker='^IXIC', start="2022-05-05", end="2022-05-06", which='Close', interval='1m'):
    #     '''
    #     # Valid periods: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
    #     # Valid intervals: 1m,2m,5m,15m,30m,60m,90m,1h,1d,5d,1wk,1mo,3mo
    #     # see https://github.com/ranaroussi/yfinance/blob/9eef951acc70121e65825ad25e7afd2edd4c3e4b/yfinance/multi.py
    #     '''
    #     def reformulate_date(df):
    #         df.columns = df.columns.map(''.join)
    #         df = df.rename_axis('Date').reset_index()
    #         return df
    #     data = yf.download(tickers = tiker, start=start, end=end, interval = interval, progress=False)
    #     data['pct_change'] = data[which].pct_change()
    #     data['log_ret'] = np.log(data[which]) - np.log(data[which].shift(1))
    #     data = reformulate_date(data)
    #     data = data[:-1]
    #     #print("most recent date is ", data.iloc[-1, 0])
    #     return data
        



class BackTest_Doublemomentum:
    def __init__(self, shortterm_momentum_settings, longterm_momentum_settings, 
                 shortterm_start_date, longterm_start_date, end_date, 
                 signal_ticker, long_ticker, short_ticker, 
                 shortterm_data_inverval, longterm_data_interval, 
                 which='Close', verbose=True):
        '''
        date format like '2022-05-04'
        
        '''
        # get data
        nasdaq = get_data(tiker=signal_ticker, start=shortterm_start_date, end=end_date, which=which, interval = shortterm_data_inverval)
        nasdaq_long = get_data(tiker=signal_ticker, start=longterm_start_date, end=end_date, which=which, interval = longterm_data_interval)
        
        tqqq = get_data(tiker=long_ticker, start=shortterm_start_date, end=end_date, which=which,interval = shortterm_data_inverval)
        sqqq =  get_data(tiker=short_ticker, start=shortterm_start_date, end=end_date, which=which, interval = shortterm_data_inverval)

        nasdaq = nasdaq[:-1]
        tqqq = tqqq[:-1]
        sqqq = sqqq[:-1]
        nasdaq_long = nasdaq_long[:-1]
        #print(start_date, end_date, nasdaq.shape[0], tqqq.shape[0], sqqq.shape[0])
        
        # add str date (min-level)
        nasdaq = self.convert_time(nasdaq)
        nasdaq_long = self.convert_time(nasdaq_long)
        tqqq = self.convert_time(tqqq)
        sqqq = self.convert_time(sqqq)
        
        # if daily or hourly, you can comment this line to save time.
        nasdaq, tqqq, sqqq = self.match_dfs(nasdaq, tqqq, sqqq)
        
        
        assert  nasdaq.shape[0] == tqqq.shape[0] == sqqq.shape[0]
        
        
        self.short_signal_df = self.convertdata_yfinance(nasdaq)
        self.long_signal_df = self.convertdata_yfinance(nasdaq_long)
        self.long_df = self.convertdata_yfinance(tqqq)
        self.short_df = self.convertdata_yfinance(sqqq)
        
        # adding log ret to asset df
        # WARNING: ret on 15:45 = given a strategy on time point 15:45, return you can get (during 15:45 ~ 16:00)
        self.long_df['log_ret'] = np.log(self.long_df['price'].shift(-1)) - np.log(self.long_df['price'])
        self.short_df['log_ret'] = np.log(self.short_df['price'].shift(-1)) - np.log(self.short_df['price'])
        # replace nan to 0
        self.long_df['log_ret'].replace(np.nan, 0, inplace=True)
        self.short_df['log_ret'].replace(np.nan, 0, inplace=True)
        assert self.long_df.shape[0] == self.short_df.shape[0] == self.short_signal_df.shape[0]
        
        
        # get signal
        self.doublemomentum = DoubleMomentum(self.short_signal_df, self.long_signal_df, shortterm_momentum_settings,longterm_momentum_settings, longterm_data_interval,  verbose)
        
        # converting signal to vector 
        self.signal_df = self.doublemomentum.signal_df # update
        self.signal_df['long_sig'] = np.squeeze(np.array([self.signal_df['doublemo_signal'] == 'long']))
        self.signal_df['short_sig'] = np.squeeze(np.array([self.signal_df['doublemo_signal'] == 'short']))
        
        
        
        # print return 
        self.return_total, self.return_long, self.return_short, self.totals, self.longs, self.shorts = self.compute_comp_return()
        self.total_ret_noncomp, self.long_ret_noncomp, self.short_ret_noncomp = self.compute_noncomp_return()
        self.total_suc, self.long_suc, self.short_suc = self.success_rate()
        # uncomment if you need this info
        # if if_plot:
        #     print(f"total componented return during {start_date} and {end_date} is: ", self.total_ret)
        #     print(f"long componented return during {start_date} and {end_date} is: ", self.return_long)
        #     print(f"short componented return during {start_date} and {end_date} is: ", self.return_short, '\n')

        #     print(f"total non-componented return during {start_date} and {end_date} is: ", self.total_ret_noncomp)
        #     print(f"long non-componented return during {start_date} and {end_date} is: ", self.long_ret_noncomp)
        #     print(f"short non-componented return during {start_date} and {end_date} is: ", self.short_ret_noncomp, '\n')


        #     print(f"total sucess rate during {start_date} and {end_date} is: ", self.total_suc)
        #     print(f"long sucess rate during {start_date} and {end_date} is: ", self.long_suc)
        #     print(f"short sucess rate during {start_date} and {end_date} is: ", self.short_suc)   

    def convert_time(self, yfi_data):
        '''
        
        
        '''
        def convert_(row):
            return row['Date'].to_pydatetime().strftime('%Y-%m-%d-%H-%M') 
        yfi_data['str_time'] = yfi_data.apply(lambda row: convert_(row), axis=1)
        return yfi_data

    def match_dfs(self, df1, df2, df3):
        dfs = [df1, df2, df3]
        all_dates = list(set(list(df1['str_time'].values)))+  list(set(list(df2['str_time'].values))) + list(set(list(df3['str_time'].values)))
        valid_dates = [date for date in all_dates if all_dates.count(date)==3]
        valid_dates = set(valid_dates)
        valid_dates = list(valid_dates)
        
        new_dfs = []
        for df in dfs:
            new_dfs.append(df[df['str_time'].isin(valid_dates)])
        

        return new_dfs[0].sort_values('Date').reset_index(drop=True), new_dfs[1].sort_values('Date').reset_index(drop=True), new_dfs[2].sort_values('Date').reset_index(drop=True)

    
    def convertdata_yfinance(self, yfi_data):
        '''
        use open during eval
        
        '''
        def convert_time(row):
            nyc = timezone('America/New_York')
            return row['Date'].to_pydatetime().astimezone(nyc)
        
        yfi_data['time'] = yfi_data.apply(lambda row: convert_time(row), axis=1)
        yfi_data.rename(columns={'Open':'price'}, inplace=True)
        return yfi_data[['time', 'price']]
    

        
    
    
    def compute_comp_return(self):
        return_totals = []
        return_longs = []
        return_shorts = []
        for i in range(self.long_df.shape[0]): #len(self.tqqq['log_ret'].values[2:])):
            return_long = np.array(self.long_df['log_ret'].values)[:i+1].dot(np.array(self.signal_df['long_sig'])[:i+1])
            return_longs.append(return_long)
            return_short = np.array(self.short_df['log_ret'].values)[:i+1].dot(np.array(self.signal_df['short_sig'])[:i+1])
            return_shorts.append(return_short)
            return_total = np.exp(return_long + return_short)
            return_totals.append(return_total)
        
        return_long = self.long_df['log_ret'].values.dot(self.signal_df['long_sig'].values)
        return_short = self.short_df['log_ret'].values.dot(self.signal_df['short_sig'].values)
        return_total = return_long + return_short
        
        
        return np.exp(return_total), np.exp(return_long), np.exp(return_short) , return_totals, return_longs, return_shorts
    
    def compute_noncomp_return(self):
        
        return_long = np.mean(np.exp(np.multiply(np.array(self.long_df['log_ret'].values), np.array(self.signal_df['long_sig']))))
        return_short = np.mean(np.exp(np.multiply(np.array(self.short_df['log_ret'].values), np.array(self.signal_df['short_sig']))))
        return_total = (return_long + return_short) / 2
        return return_total, return_long, return_short
        
        
        
    def success_rate(self):
        long_arr = np.array(self.long_df['log_ret'].values)
        short_arr = np.array(self.short_df['log_ret'].values)
        try:
            rate_long = (np.array(long_arr > 0).astype(int).dot(self.signal_df['long_sig'].values.astype(int))) / sum(self.signal_df['long_sig'].values)
        except:
            rate_long = 1
        try:
            rate_short = (np.array(short_arr > 0).astype(int).dot(self.signal_df['short_sig'].values.astype(int))) / sum(self.signal_df['short_sig'].values)
        except:
            rate_short = 1
        try:
            total_rate = ((np.array(long_arr > 0).astype(int).dot(np.array(self.signal_df['long_sig']).astype(int))) + (np.array(short_arr > 0).astype(int).dot(np.array(self.signal_df['short_sig']).astype(int))) ) / (sum(self.signal_df['long_sig'])+sum(self.signal_df['short_sig']))
        except:
            total_rate = 1
        return total_rate, rate_long, rate_short
    


    # def __init__(self, shortterm_momentum_settings, longterm_momentum_settings, 
    #              shortterm_start_date, longterm_start_date, end_date, 
    #              signal_ticker, long_ticker, short_ticker, 
    #              shortterm_data_inverval, longterm_data_interval, 
    #              which='Close', verbose=True): 

class RollingBackTest_DoubleMomentum():
    # '''
    # WARINING:
    # for 1m: due to yahoo finance data restriction, must be within the last 30 days
    # - 
    # '''
    def __init__(self, api_setting:APISettings(),
                 shortterm_momentum_settings, longterm_momentum_settings,  
                 start_date, testing_length, gap, shortterm_datainterval, longterm_datainterval,
                 signal_ticker, long_ticker, short_ticker,
                 verbose):
        
        self.start_date = start_date
        self.testing_length = testing_length
        self.gap = gap
        self.signal_ticker = signal_ticker
        self.long_ticker = long_ticker
        self.short_ticker = short_ticker
        self.shortterm_datainterval = shortterm_datainterval
        self.longterm_datainterval = longterm_datainterval
        self.verbose =  verbose
        self.shortterm_momentum_settings =  shortterm_momentum_settings
        self.longterm_momentum_settings = longterm_momentum_settings
        self.api = REST(api_setting.API_KEY_ID, api_setting.SECRET_KEY, api_setting.ENDPOINT)
        
        self.total_rets, self.long_rets, self.short_rets, self.rets_noncomp = self.testing()

    def create_longterm_startdate(self, shortterm_startdate:str, longterm_interval:str):
        # define long term back days
        if longterm_interval in ['15m', '30m']:
            backdays = 29
        elif longterm_interval == '1h':
            backdays = 59
        elif longterm_interval == '1d':
            backdays = 100
        shortterm = datetime.strptime(shortterm_startdate, '%Y-%m-%d')
        longterm = shortterm - timedelta(days=backdays)
        longterm_str = longterm.strftime('%Y-%m-%d')
        return longterm_str
        
        
    def testing(self):
        start_dates, end_dates = self.create_rolling_dates()
        print("start testing...")
        test_returns = []
        long_returns = []
        short_returns = []
        rets_noncomp = []
        total_sucs = []
        for start_date, end_date in tqdm(zip(start_dates, end_dates), total=len(end_dates)):


            backtest = BackTest_Doublemomentum(self.shortterm_momentum_settings, self.longterm_momentum_settings,
                                                start_date, self.create_longterm_startdate(start_date, self.longterm_datainterval), end_date,
                                                self.signal_ticker, self.long_ticker, self.short_ticker,
                                                self.shortterm_datainterval, self.longterm_datainterval, 
                                                verbose=self.verbose)
            test_returns.append(backtest.return_total)
            long_returns.append(backtest.return_long)
            short_returns.append(backtest.return_short)
            rets_noncomp.append(backtest.total_ret_noncomp)
            total_sucs.append(backtest.total_suc)
            # try:
            #     backtest = BackTest_Doublemomentum(self.shortterm_momentum_settings, self.longterm_momentum_settings,
            #                                        start_date, self.create_longterm_startdate(start_date), end_date,
            #                                        self.signal_ticker, self.long_ticker, self.short_ticker,
            #                                        self.shortterm_datainterval, self.longterm_datainterval, 
            #                                        verbose=self.verbose)
            #     test_returns.append(backtest.return_total)
            #     long_returns.append(backtest.return_long)
            #     short_returns.append(backtest.return_short)
            #     rets_noncomp.append(backtest.total_ret_noncomp)
            #     total_sucs.append(backtest.total_suc)
                
            # except:
            #     test_returns.append(1)
            #     long_returns.append(1)
            #     short_returns.append(1)  
            #     rets_noncomp.append(1)
            #     total_sucs.append(0.5)
        
        return  test_returns, long_returns, short_returns, rets_noncomp
        
    
    
    def create_rolling_dates(self):
        start_testing = self.start_date

        nyc = timezone('America/New_York')
        #now = datetime.today().astimezone(nyc)
        days_before = datetime.today() - timedelta(self.testing_length)
        days_before_str = days_before.astimezone(nyc).strftime('%Y-%m-%d')
        calendar = self.api.get_calendar(start=start_testing, end=days_before_str) # get the next closet trading day 
        calender_date_str = [date.date for date in calendar] # .strftime('%Y-%m-%d')
        calender_date_str = calender_date_str[0::self.gap]
        starts = []
        ends = []
        for date in calender_date_str:
            starts.append(date.strftime('%Y-%m-%d'))
            end = date + timedelta(days=self.testing_length)
            ends.append(end.strftime('%Y-%m-%d'))
        
        return (starts, ends)


from .momentum import Momentum
from .__main__ import Strategy
from array import array
from .computation import dif, ema
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

from pytz import timezone
from datetime import datetime, timedelta

from copy import deepcopy

class DoubleMomentum(Strategy):
    '''
    - trading using short term momentum(min) but also considering long term momentum(day)
    
    '''
    def __init__(self, shortterm_signal_df:pd.DataFrame, longterm_signal_df:pd.DataFrame, shortterm_momentum_settings, longterm_momentum_settings, longterm_data_interval, verbose=True):
        '''
        prices :
            - pandas dataframe, at least need two variables: timestamp , Close price named ['Close']
            - prices = signal prices (IXIC)
            - long asset: may not be TQQQ
            - short asset: can only be SQQQ
        '''
        super().__init__('doublemomemtum')
        self.big_momemtum_setting = longterm_momentum_settings
        self.small_momentum_setting = shortterm_momentum_settings
        self.verbose = verbose
        self.shortterm_signal_df = shortterm_signal_df
        self.longterm_signal_df = longterm_signal_df
        self.longterm_data_interval = longterm_data_interval

        
        
        # create momentum
        self.shortterm_momentum = Momentum(self.shortterm_signal_df, shortterm_momentum_settings, False)
        self.longterm_momentum = Momentum(self.longterm_signal_df, longterm_momentum_settings, False)
        
        # obtain signal df
        self.short_signal_df = self.shortterm_momentum.signal_df
        self.original_signal_df = deepcopy(self.short_signal_df)
        self.long_signal_df = self.longterm_momentum.signal_df
        
        # obtain final
        self.signal_df = self.double_momentum()
        
        # if verbose
        if verbose:
            self.plot_strategy()
            self.plot_original()
    
    def plot_strategy(self):
        figure_size  = (20, 10)
        fig, ax = plt.subplots(figsize=figure_size)
        ax.plot(self.signal_df['price'], label="signal price")
        #if emas != None:
        #ax.plot(self.signal_df['ema'], label=f'ema{self.momemtum_settings.ema_smooth_factor} of signal price')
        # print long
        ax.vlines(x= np.array(self.signal_df[self.signal_df['doublemo_signal'] == 'long'].index), 
                  ymin=np.min(self.signal_df['price']), 
                  ymax=np.max(self.signal_df['price']), 
                  color='g', linestyle='-.')
        # print short
        ax.vlines(x= np.array(self.signal_df[self.signal_df['doublemo_signal'] == 'short'].index), 
                  ymin=np.min(self.signal_df['price']), 
                  ymax=np.max(self.signal_df['price']), 
                  color='r', linestyle='-.')
        ax.title.set_text('Double Momentum')
        ax.legend()
        
        
    def plot_original(self):
        figure_size  = (20, 10)
        fig, ax = plt.subplots(figsize=figure_size)
        ax.plot(self.original_signal_df['price'], label="signal price")
        #if emas != None:
        #ax.plot(self.signal_df['ema'], label=f'ema{self.momemtum_settings.ema_smooth_factor} of signal price')
        # print long
        ax.vlines(x= np.array(self.original_signal_df[self.original_signal_df['signal'] == 'long'].index), 
                  ymin=np.min(self.original_signal_df['price']), 
                  ymax=np.max(self.original_signal_df['price']), 
                  color='g', linestyle='-.')
        # print short
        ax.vlines(x= np.array(self.original_signal_df[self.original_signal_df['signal'] == 'short'].index), 
                  ymin=np.min(self.original_signal_df['price']), 
                  ymax=np.max(self.original_signal_df['price']), 
                  color='r', linestyle='-.')
        ax.title.set_text('Single Momentum')
        ax.legend()
        
        

        
    def double_momentum(self):
        '''
        according to long term momentum signal, edit short term momentum signal
            - edit will make directly on self.shortterm_momentum.signal_df
        '''
        # adding new var
        self.short_signal_df['daytime'] = self.short_signal_df.apply(lambda row: self.convert_str(row), axis=1)
        self.long_signal_df['daytime'] = self.long_signal_df.apply(lambda row: self.convert_str(row), axis=1)
        
        # 
        self.short_signal_df['doublemo_signal'] = self.short_signal_df.apply(lambda row: self.convert_signal(row), axis=1)
        return self.short_signal_df
    
    def convert_signal(self, row):
        daytime = row['daytime']
        shorterm_sig = row['signal']
        #print(daytime, self.long_signal_df['daytime'].values[-1])
        longterm_sig = self.long_signal_df[self.long_signal_df['daytime'] == daytime]['signal'].values
        
        #print(longterm_sig)
        #print(longterm_sig, shorterm_sig)
        if longterm_sig != shorterm_sig:
            return 'empty'
        else:
            return shorterm_sig
        
        
    def convert_str(self, row):
        '''
        Can only be used when:
            - longterm = day
            - shortterm = intraday
        '''
        if self.longterm_data_interval == '1d':
            return row['time'].strftime('%Y-%m-%d') 
        elif self.longterm_data_interval == '1h':
            return row['time'].strftime('%Y-%m-%d-%H')
        else:
            raise ValueError


            
            
            
                
        
        # def create_signal_with_time(price_df, momentum, momentum_name:str):
        #     nyc = timezone('America/New_York')
        #     def add_str_time(row):
        #         return row['time'].to_pydatetime().astimezone(nyc).strftime('%Y-%m-%d')
            
        #     signal_df = pd.DataFrame({'time':price_df['Date'][1:-1], 
        #                             'long':momentum.signals['long'],
        #                             'short':momentum.signals['short'],
        #                             'empty':momentum.signals['empty']})
        #     signal_df['time_str'] = signal_df.apply(lambda row: add_str_time(row), axis=1)
        #     return signal_df
        
        
        
        
            
        

        
        
        
        
    
    


from .__main__ import Strategy
from array import array
from .computation import dif, ema
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

from datetime import datetime, timedelta
from pytz import timezone

class Momentum_settings:
    '''
    if you found it is more likely to enter "long stage"
        - you should set higher threshold for short, but lower threshold for long
    '''
    long_buy_thres = 0
    long_sell_thres = 0
    short_buy_thres = 6
    short_sell_thres = 6
    ema_smooth_factor = 5 # long-term indicator as ema smooth factor
    # if larger, slower track of current price, more consistent
    dif_smooth_factor = 2 # price smooth factor for computing momentum
    
    
    

class Momentum(Strategy):
    '''
    ema enhanced momentum strategy
    '''
    def __init__(self, signal_df, momemtum_settings, verbose=False, only_morning=False):
        '''
        Purpose:
            input data, output signal(long? short? empty?) to the corresponding **time period** 
                - **time period** represents as its start time point, datetime.datetime
        
        Args:
            signal_df: contains 2 columns:
                - ['time']: datetime.datetime
                - ['price']: price wrt the time point
            
            momentum_setting:
        
        WARINING:
            - when backtest, price = the price of corresponding timepoint (use Open)
            - when real-time, price = the price of corresponding timepoint + 1 interval (use Close)
        '''
        
        super().__init__('momentum')
        self.only_morning = only_morning
        self.signal_df = signal_df
        self.momemtum_settings = momemtum_settings
        self.verbose = verbose
        #self.signal_df['pydate'] = signal_df.apply(lambda row: self.convert_time(row, 'Date'), axis=1)
        #self.long_asset_prices = long_asset_df['Close'].values
        
        ####### main process
        # create long term indicator
        self.signal_df['ema']  = self.long_term_momentum()
        # create short term momentum 
        differentials = self.compute_momentum(self.momemtum_settings.dif_smooth_factor)
        self.signal_df['dif'] = np.nan
        self.signal_df['dif'][1:] = differentials
        # create signal
        self.signal_df['signal'] = signal_df.apply(lambda row: self.create_signal_row(row), axis=1)
        
        if verbose:
            self.plot_strategy()
            
 
    
    def plot_strategy(self):


        figure_size  = (20, 10)
        fig, ax = plt.subplots(figsize=figure_size)
        ax.plot(self.signal_df['price'], label="signal price")
        #if emas != None:
        ax.plot(self.signal_df['ema'], label=f'ema{self.momemtum_settings.ema_smooth_factor} of signal price')
        # print long
        ax.vlines(x= np.array(self.signal_df[self.signal_df['signal'] == 'long'].index), 
                  ymin=np.min(self.signal_df['price']), 
                  ymax=np.max(self.signal_df['price']), 
                  color='g', linestyle='-.')
        # print short
        ax.vlines(x= np.array(self.signal_df[self.signal_df['signal'] == 'short'].index), 
                  ymin=np.min(self.signal_df['price']), 
                  ymax=np.max(self.signal_df['price']), 
                  color='r', linestyle='-.')
        
        ax.legend()
        #pass
    
    
    def convert_time(self, row, colname='Date'):
        nyc = timezone('America/New_York')
        return row[colname].to_pydatetime().astimezone(nyc)
    
    def create_signal_row(self, row):
        price = row['price']
        dif = row['dif']
        ema = row['ema']
        time = row['time']
        
        if self.only_morning == False:
            if dif == np.nan:
                return 'empty'
            else:
                if dif > self.momemtum_settings.long_buy_thres and price > ema: # long
                    return 'long'
                elif dif < -self.momemtum_settings.short_buy_thres and price < ema:
                    return 'short'
                else:
                    return 'empty'
        else:
            if dif == np.nan:
                return 'empty'
            else:
                if dif > self.momemtum_settings.long_buy_thres and price > ema and int(time.strftime('%H')) < 10: # long
                    return 'long'
                elif dif < -self.momemtum_settings.short_buy_thres and price < ema and int(time.strftime('%H')) < 10:
                    return 'short'
                else:
                    return 'empty'
            
            

        
        
    def long_term_momentum(self):
        '''
        use ema20 as default for minites trade
        '''
        return ema(self.signal_df['price'], self.momemtum_settings.ema_smooth_factor)
    
    
    def compute_momentum(self, dif_sf, focal_row_nums=[]):
        '''
        1st differential
        
        - possible improvement:
            - 1st differential is related to the abosolute index value
            - so divided by index value  / self.prices[-1]
        '''
        ma_dif = dif(ema(self.signal_df['price'], dif_sf)) 
        assert len(ma_dif) == self.signal_df.shape[0] - 1
        

        if self.verbose:
            
            # #if you want to plot prices in a seperated plot, uncomment it.
            # fig, ax = plt.subplots(figsize=figure_size)
            # ax.plot(ori_data, label="Price")
            
            # if len(focal_row_nums) != 0:
            #     for focal_row_num in focal_row_nums:
            #         ax.vlines(ymin=np.min(ori_data), ymax=np.max(ori_data), x=focal_row_num, color='r', linestyle='-.')
            # ax.legend()
            
            figure_size  = (20, 10)
            fig, ax = plt.subplots(figsize=figure_size)
            #ax.plot(dif, label="dif: divergence")
            ax.hlines(xmin=0, xmax=ma_dif.shape[0], y = 0, color='r', linestyle='-.')
            #assert dif.shape[0] == ori_data.shape[0] == ma_dif.shape[0]
            for focal_row_num in focal_row_nums:
                ax.vlines(ymin=np.min(ma_dif), ymax=np.max(ma_dif), x=focal_row_num, color='r', linestyle='-.')
            ax.plot(ma_dif, label="MA of delta price: denoised(smoothed) divergence")
            #ax.plot(ema_ema, label="EMA of EMA")
            #ax.plot(macd, label="macd")
            ax.legend()
        
        return ma_dif
        
        
        
    

        
        
        
### old
                
    
    # def create_signal(self):
        
        
    #     # no data for first differential for 1st data point
    #     emas = self.long_term_indicator
    #     prices = self.signal_df['Close'].values[1:]
    #     times = self.signal_df['pydate'].values[1:]
    #     emas = emas[1:]
        
    #     # create signal
    #     delta_diver = self.differentials
    #     long = []
    #     short = []
    #     empty = []
    #     for i, value in enumerate(delta_diver):
            
    #         if i==0: # when time =0, only consider when to buy
    #             if value > self.momemtum_settings.long_buy_thres: # buy long 
    #                 long.append(1)
    #                 short.append(0)
    #                 empty.append(0)
    #             elif value < - self.momemtum_settings.short_buy_thres: # buy short
    #                 long.append(0)
    #                 short.append(1)
    #                 empty.append(0)
    #             else:
    #                 long.append(0)
    #                 short.append(0)
    #                 empty.append(1)
    #         else: # when time >0, obtain prev status and consider current operation
    #             if long[i-1] == 1: # if prev hold long, consider when to sell long
    #                 if value < self.momemtum_settings.long_sell_thres: # if <= long_sell_thres, sell
    #                     if value < - self.momemtum_settings.short_buy_thres: # if buy short instead? < -thres
    #                         long.append(0)
    #                         short.append(1)
    #                         empty.append(0)
    #                     else:
    #                         long.append(0)
    #                         short.append(0)
    #                         empty.append(1)   
    #                 else: # continue to hold
    #                     long.append(1)
    #                     short.append(0)
    #                     empty.append(0)
                        
    #             elif short[i-1] == 1: # if prev hold short
    #                 if value > - self.momemtum_settings.short_sell_thres: # sell short
    #                     if value > self.momemtum_settings.long_buy_thres: # buy long instead?
    #                         long.append(1)
    #                         short.append(0)
    #                         empty.append(0)
    #                     else:
    #                         long.append(0)
    #                         short.append(0)
    #                         empty.append(1)
    #                 else: # continue to hold
    #                     long.append(0)
    #                     short.append(1)
    #                     empty.append(0)
                
    #             elif empty[i-1] == 1: # prev did not hold anything
    #                 if value > self.momemtum_settings.long_buy_thres:
    #                     long.append(1)
    #                     short.append(0)
    #                     empty.append(0)
    #                 elif value < - self.momemtum_settings.short_buy_thres:
    #                         long.append(0)
    #                         short.append(1)
    #                         empty.append(0)
    #                 else:
    #                     long.append(0)
    #                     short.append(0)
    #                     empty.append(1)

    #         if emas[i] > prices[i]: # go down trend
    #             if long[i] == 1:
    #                 long[i] = 0
    #                 empty[i]=1
    #         elif emas[i] <= prices[i]:
    #             if short[i] == 1:
    #                 short[i] = 0
    #                 empty[i]=1
                             
                
        
    #     assert len(prices) == len(emas) == len(long)
    #     for long_, short_, empty_ in zip(long, short, empty):
    #         assert ((long_ + short_ + empty_) == 1)

    #     signals = {'long':long, 'short':short, 'empty':empty, 'signal_time':times}
        
        
        
        # return signals # last signal is useless   


#!/usr/bin/env python3





# document of alpaca api: https://github.com/alpacahq/alpaca-trade-api-python
# transaction cost: https://www.webull.com/pricing

# 200 per minute query limit alpaca has in place.
import sys
from alpaca_trade_api.rest import REST, TimeFrame
from alpaca_trade_api.stream import Stream
import alpaca_trade_api as tradeapi

import time
import numpy as np
from datetime import datetime, timedelta
from pytz import timezone


from .strategy.momentum import Momentum
import yfinance as yf
from .utils.tradebot import RD, EMA
from .utils.data import get_data




def sell_everything():
    '''
    focal_ticker:
        - str
    refer to: https://github.com/alpacahq/alpaca-trade-api-python/blob/a9055b9b55b4da9c39d849d21e73af5601fbe832/alpaca_trade_api/rest.py
    '''
    existing_positions = api.list_positions()
    start_time = time.time()
    for position in existing_positions:
        api.submit_order(
            symbol=position.symbol,
            qty = position.qty,
            side='sell',
            type='market',
            time_in_force='day'
        )
        
    ##### double check sell out
    total = 0
    while True:
        if len(api.list_orders(status='open', side='sell')) == 0:
            break
        else:
            time.sleep(0.5)
            total += 1
            assert total < 10
            
    total = 0        
    while True:
        position_lst = api.list_positions()
        if len(position_lst) == 0:
            break
        else:
            time.sleep(0.5)
            total += 1
            assert total < 10
    
    total = 0
    # while True:
    #     try:
    #         assert len(existing_positions) == 0 
    #         break
    #     except:
    #         time.sleep(0.5)
    #         total += 1
    #         assert total < 10
            
            
    # assert len(existing_positions) == 0 
    trans_time = time.time() - start_time
    print(f"Sell everything ||| Trans time:"+"{:.2f} seconds".format(round(trans_time, 2)))
    
    
    
    
def full_sell(focal_ticker):
    '''
    focal_ticker:
        - str
    refer to: https://github.com/alpacahq/alpaca-trade-api-python/blob/a9055b9b55b4da9c39d849d21e73af5601fbe832/alpaca_trade_api/rest.py
    '''
    focal_position = api.get_position(focal_ticker)
    sell_quantity = focal_position.qty
    start_time = time.time()
    api.submit_order(
        symbol=focal_ticker,
        qty = sell_quantity,
        side='sell',
        type='market',
        time_in_force='day'
    )
    
    # double check
    while True:
        if len(api.list_orders(status='open', side='sell')) == 0:
            break
        else:
            time.sleep(0.4)
        
    while True:
        try:
            api.get_position(focal_ticker) # if ==0, not exist error
            time.sleep(0.001)
        except:
            break
    trans_time = time.time() - start_time
    print(f"Sell: {focal_ticker} ||| Quantity: {sell_quantity} ||| Trans time:"+"{:.2f} seconds".format(round(trans_time, 2)))



def full_buy(focal_ticker, bp_portion):
    '''
    balance:
        - float
    focal_ticker:
        - str
    TODO: get_postion function has delayed.....not working
    Attention: Buying power is fluctuating since equity value fluctuate
    '''
    bp = float(api.get_account().buying_power)
    # get_qty
    # try:
    #     old_qty = float(api.get_position(focal_ticker).qty)
    # except:
    #     old_qty = 0
    if bp > 1: # 
        start_time = time.time()
        api.submit_order(
            symbol=focal_ticker,
            notional = bp*bp_portion, # sometimes 
            side='buy',
            type='market',
            time_in_force='day'
        )
        while True:
            if len(api.list_orders(status='open', side='buy')) == 0:
                break
            else:
                time.sleep(0.4)
        # # get qty
        # try:
        #     new_qty = float(api.get_position(focal_ticker).qty)
        # except:
        #     new_qty = 0
        # # make sure order complete
        # while True:
        #     if new_qty > old_qty:
        #         break
        #     else:
        #         time.sleep(0.001)
        trans_time = time.time() - start_time
        
        print(f"Buy: {focal_ticker} ||| Ammount: ${bp*bp_portion} ||| Trans time:"+"{:.2f} seconds".format(round(trans_time, 2)))
    
    else:
        pass        
            
def my_submit_order(sig, long_ticker,  short_ticker, bp_portion):
    '''
    run once
        - transact once
    long_signal: 
        - bool
        - if True, 
            - if hold short
                - sell short and buy long asset
            - if hold long
                - nothing to do
    
    long_ticker:
        - str
    '''
    #print(sig)
    # get position quantity
    try:
        short_position = float(api.get_position(short_ticker).qty)
    except:
        short_position = 0
    try:
        long_position = float(api.get_position(long_ticker).qty)
    except:
        long_position = 0
    
    action_lst = []
    # decide operation: sell something, buy something
    # long_sig = signals['long'][-1]
    # short_sig = signals['short'][-1]
    #empty_sig = signals['empty'][-1]
    
    
    if (short_position==0) and (long_position==0): # empty
        if sig == 'long':# buy long
            action_lst.append('buy long')
        elif sig == 'short':
            action_lst.append('buy short')

    else:
        if short_position > 0: # consider hold or sell short
            if sig != 'short': # sell short
                action_lst.append('sell short')
                if sig == 'long':
                    action_lst.append('buy long')
        elif long_position > 0: # consider hold or sell long
            if sig != 'long':
                action_lst.append('sell long')
                if sig == 'short':
                    action_lst.append('buy short')
    
    # execution
    for act in action_lst:
        if act == 'buy long':
            full_buy(long_ticker, bp_portion)
        elif act == 'buy short':
            full_buy(short_ticker, bp_portion)
        elif act == 'sell long':
            full_sell(long_ticker)
        elif act == 'sell short':
            full_sell(short_ticker)
            
            
           
    time.sleep(0.4)          
    # renew position qty    
    try:
        short_position = float(api.get_position(short_ticker).qty)
        time.sleep(0.4) 
    except:
        short_position = 0
    try:
        long_position = float(api.get_position(long_ticker).qty)
    except:
        long_position = 0
    # must be each of the case
    # error : long position 0, short position 0, long signal 0
    print(f"Current sig: {sig}. Acted: {action_lst}. HOLD at {datetime.today().astimezone(nyc)}: long position {long_position}, short position {short_position} \n")

    # you cannot hold long and short at the same time.
    if ((long_position>0) and (short_position>0)):
        print("Warining: hold long and short at the same time.")
        sell_everything()
        while True:
            if not ((long_position>0) and (short_position>0)):
                break
            else:
                print("Warining: hold long and short at the same time.")
                time.sleep(5)
    assert (not ((long_position>0) and (short_position>0)))

    
    

# def get_data(tiker='^IXIC', start="2022-05-05", which='Close', interval='1m'):
#     '''
#     # Valid periods: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
#     # Valid intervals: 1m,2m,5m,15m,30m,60m,90m,1h,1d,5d,1wk,1mo,3mo
#     # see https://github.com/ranaroussi/yfinance/blob/9eef951acc70121e65825ad25e7afd2edd4c3e4b/yfinance/multi.py
#     '''
#     def reformulate_date(df):
#         df.columns = df.columns.map(''.join)
#         df = df.rename_axis('Date').reset_index()
#         return df
#     data = yf.download(tickers = tiker, start=start, interval = interval, progress=False)
#     #data['pct_change'] = data[which].pct_change()
#     #data['log_ret'] = np.log(data[which]) - np.log(data[which].shift(1))
#     data = reformulate_date(data)
#     #print("most recent date is ", data.iloc[-1, 0])
#     return data



def convertdata_yfinance(yfi_data):
    '''
    use 'Close' data in Live
    '''
    def convert_time(row):
        nyc = timezone('America/New_York')
        return row['Date'].to_pydatetime().astimezone(nyc)
    
    yfi_data['time'] = yfi_data.apply(lambda row: convert_time(row), axis=1)
    yfi_data.rename(columns={'Close':'price'}, inplace=True)
    return yfi_data[['time', 'price']]


def daily(trade_freq = '1m', signal_ticker='^IXIC', long_ticker='TQQQ', short_ticker='SQQQ', bp_portion=1/2):
    # check account status
    account = api.get_account()
    assert account.status == 'ACTIVE'
    # get today
    #now = datetime.today().astimezone(nyc)
    today_str = datetime.today()+ timedelta(1)
    today_str = today_str.astimezone(nyc).strftime('%Y-%m-%d')
    # get market open/close datetime 
    market_open = now.replace(
        hour=calendar.open.hour,
        minute=calendar.open.minute,
        second=0
    )
    market_open = market_open.astimezone(nyc)
    market_close = now.replace(
        hour=calendar.close.hour,
        minute=calendar.close.minute,
        second=0
    )
    market_close = market_close.astimezone(nyc)
    # get current datetime
    #current_dt = datetime.today().astimezone(nyc)
    # how much the time has pass since market open
    #since_market_open = current_dt - market_open
    # how much the time remain before market close
    #remain_market_open = market_close - current_dt
    if trade_freq == '1m':
        seconds_per_turn = 60
        time_delta = 5
    if trade_freq == '2m':
        seconds_per_turn = 120
        time_delta = 5
    if trade_freq == '1h':
        seconds_per_turn = 60*60
        time_delta = 100
    # today's start buying power
    daily_start_all_value = float(api.get_account().equity)
    old_length_of_price = 0
    
    suspend_secs = 5 # suspend for a loop try
    
    
    #### trading loop
    while True:
        start_time = time.time()
        # get signal
        
        while True:
            try:
                days_before = datetime.today() - timedelta(time_delta) # 3 for ...weekend
                data_start_date = days_before.astimezone(nyc).strftime('%Y-%m-%d') # to make sure ema is good enough
                df = get_data(tiker=signal_ticker, start = data_start_date, end=today_str, which='Close', interval=trade_freq)
                df = df[:-1] # remove the last line (its the varying live price)
                length_of_price = df.shape[0]
                lastdata_timestamp = df.iloc[-1, 0]
                break
            except:
                print("Getting data meets error, sleep 5 seconds and try again.")
                time.sleep(5)
                
        
        if length_of_price > old_length_of_price: # get updated price info
            print('Latest data point at:', lastdata_timestamp)
            # formulate df 
            new_df = convertdata_yfinance(df)
            # get current signal
            momentum = Momentum(new_df, Momentum_settings(), False, False)
            sigs = momentum.signal_df['signal'].values[-1] # a string

    
            my_submit_order(sigs, long_ticker, short_ticker, bp_portion)

            old_length_of_price = length_of_price
            time.sleep(0.5)
        
        current_dt = datetime.today().astimezone(nyc)
        #since_market_open = current_dt - market_open
        remain_market_open = market_close - current_dt
        remain_market_open = remain_market_open.total_seconds()
        
        
        ##### once 
        #print('remain trade time today:', remain_market_open.seconds)
        if remain_market_open <= 20:
            sell_everything()
            print(f"############### Trade at date {today_str} ends ##############")

            break
        
        
        ##### Loss Security Wall ##### 
        # once loss 10%, stop trading today
        all_value = float(api.get_account().equity)
        if all_value/daily_start_all_value < (1-0.1): # max loss = 10%
            break
 
        
        
        if (suspend_secs - (time.time() - start_time)) > 0:
            time.sleep(suspend_secs - (time.time() - start_time)) # make sure suspend 5 seconds 
        # Todo: this may not be good. Since purchasing and selling take extra time. This should be less than 60 seconds.








####
# 
#
#
#
#
####
    
class Momentum_settings:
    '''
    if you found it is more likely to enter "long stage"
        - you should set higher threshold for short, but lower threshold for long
    '''
    long_buy_thres = 1
    long_sell_thres = 2
    short_buy_thres = 1
    short_sell_thres = 2
    ema_smooth_factor = 30 # long-term indicator as ema smooth factor
    # if larger, slower track of current price, more consistent
    dif_smooth_factor = 5 # price smooth factor for computing momentum
    

def main():

    API_KEY_ID = "yours"
    SECRET_KEY = "yours"
    ENDPOINT = 'https://paper-api.alpaca.markets'

    if API_KEY_ID == 'yours':
        print("Please replcacing API KEYs with yours")
    
    api = REST(API_KEY_ID, SECRET_KEY, ENDPOINT)
    
    # hyperparam
    trade_freq = '1m' # change this 
    long_ticker = 'TQQQ'
    short_ticker = 'SQQQ'
    # stop_after_open = 0
    # stop_before_close = 0 # 2 * trade_freq
    bp_portion = 1/4
    
    
    
    
    
    
     
    
    if trade_freq == '1m':
        seconds_per_turn = 60
    if trade_freq == '15m':
        seconds_per_turn = 60*15
    if trade_freq == '1h':
        seconds_per_turn = 60*60
    
    while True:
        '''
        end trading at 16:05
        start trading at 09:25
        '''
        nyc = timezone('America/New_York')
        now = datetime.today().astimezone(nyc)
        today_str = datetime.today().astimezone(nyc).strftime('%Y-%m-%d')
        # if today is a trading day.. return today
        calendar = api.get_calendar(start=today_str, end=today_str)[0] # get the next closet trading day 
        # judge if today is a trading day
        is_today_tradingday = (today_str == datetime.strptime(str(calendar.date), '%Y-%m-%d %H:%M:%S').strftime('%Y-%m-%d'))
        # get market open/close datetime 
        market_open_time = now.replace(
            hour=calendar.open.hour,
            minute=calendar.open.minute,
            second=0
        )
        market_open_time = market_open_time.astimezone(nyc)
        market_close_time = now.replace(
            hour=calendar.close.hour,
            minute=calendar.close.minute,
            second=0
        )
        market_close_time = market_close_time.astimezone(nyc)
        # how much the time has pass since market open
        since_market_open = now - market_open_time # seconds
        since_market_open = since_market_open.total_seconds()
        # how much the time remain before market close
        remain_market_open = market_close_time - now # seconds
        remain_market_open = remain_market_open.total_seconds()
        # if trading time
        # start before open, stop before close
        is_trading_running =  (since_market_open > 60*1) and (remain_market_open > 60*3) #(now >= market_open_time) and (now <= market_close_time) and (remain_market_open.seconds // seconds_per_turn > stop_before_close) and (since_market_open.seconds // seconds_per_turn >= stop_after_open)

        if is_today_tradingday and is_trading_running:
            daily(trade_freq = trade_freq, signal_ticker='^IXIC', long_ticker=long_ticker, short_ticker=short_ticker, bp_portion=bp_portion) 
        else:
            time.sleep(10)
            print(f"Sleep at {now}...Waiting for market open...\n")
   



from typing import Dict
from abc import abstractmethod
from array import array


class Strategy:
    '''
    Base class of strategy
    '''
    
    def __init__(self, name: str):
        self.name = name



    
    @abstractmethod
    def create_signal(self) -> Dict:
        '''
        create signal: 3 list of signal
            - long
            - short
            - empty
        
        if 1 = do this action **after observing the data in this timestamp**
        defalut = 0
        '''
        raise NotImplementedError
        

    def backtest(self) -> None:
        '''
        Non rolling back test
        
        print:
            - success rate
            - total return
        '''
        raise NotImplementedError
        
        
    



        
        
        
        
        
    


# MyTT 麦语言-通达信-同花顺指标实现     https://github.com/mpquant/MyTT
# MyTT高级函数验证版本：               https://github.com/mpquant/MyTT/blob/main/MyTT_plus.py
# Python2老版本pandas特别的MyTT：      https://github.com/mpquant/MyTT/blob/main/MyTT_python2.py 
# V2.1  2021-6-6   新增 BARSLAST函数 SLOPE,FORCAST线性回归预测函数
# V2.3  2021-6-13  新增 TRIX,DPO,BRAR,DMA,MTM,MASS,ROC,VR,ASI等指标
# V2.4  2021-6-27  新增 EXPMA,OBV,MFI指标, 改进SMA核心函数(核心函数彻底无循环)
# V2.7  2021-11-21 修正 SLOPE,BARSLAST,函数,新加FILTER,LONGCROSS, 感谢qzhjiang对SLOPE,SMA等函数的指正
# V2.8  2021-11-23 修正 FORCAST,WMA函数,欢迎qzhjiang,stanene,bcq加入社群，一起来完善myTT库
# V2.9  2021-11-29 新增 HHVBARS,LLVBARS,CONST, VALUEWHEN功能函数
# V2.92 2021-11-30 新增 BARSSINCEN函数,现在可以 pip install MyTT 完成安装   
# V3.0  2021-12-04 改进 DMA函数支持序列,新增XS2 薛斯通道II指标
# V3.1  2021-12-19 新增 TOPRANGE,LOWRANGE一级函数
  

#以下所有函数如无特别说明，输入参数S均为numpy序列或者列表list，N为整型int
#应用层1级函数完美兼容通达信或同花顺，具体使用方法请参考通达信

import numpy as np; import pandas as pd

#------------------ 0级：核心工具函数 --------------------------------------------      
def RD(N,D=3):   return np.round(N,D)        #四舍五入取3位小数 
def RET(S,N=1):  return np.array(S)[-N]      #返回序列倒数第N个值,默认返回最后一个
def ABS(S):      return np.abs(S)            #返回N的绝对值
def LN(S):       return np.log(S)            #求底是e的自然对数,
def POW(S,N):    return np.power(S,N)        #求S的N次方
def SQRT(S):     return np.sqrt(S)           #求S的平方根
def MAX(S1,S2):  return np.maximum(S1,S2)    #序列max
def MIN(S1,S2):  return np.minimum(S1,S2)    #序列min
def IF(S,A,B):   return np.where(S,A,B)      #序列布尔判断 return=A  if S==True  else  B


def REF(S, N=1):          #对序列整体下移动N,返回序列(shift后会产生NAN)    
    return pd.Series(S).shift(N).values  

def DIFF(S, N=1):         #前一个值减后一个值,前面会产生nan 
    return pd.Series(S).diff(N).values     #np.diff(S)直接删除nan，会少一行

def STD(S,N):             #求序列的N日标准差，返回序列    
    return  pd.Series(S).rolling(N).std(ddof=0).values     

def SUM(S, N):            #对序列求N天累计和，返回序列    N=0对序列所有依次求和         
    return pd.Series(S).rolling(N).sum().values if N>0 else pd.Series(S).cumsum().values  

def CONST(S):             #返回序列S最后的值组成常量序列
    return np.full(len(S),S[-1])
  
def HHV(S,N):             #HHV(C, 5) 最近5天收盘最高价        
    return pd.Series(S).rolling(N).max().values     

def LLV(S,N):             #LLV(C, 5) 最近5天收盘最低价     
    return pd.Series(S).rolling(N).min().values    
    
def HHVBARS(S,N):         #求N周期内S最高值到当前周期数, 返回序列
    return pd.Series(S).rolling(N).apply(lambda x: np.argmax(x[::-1]),raw=True).values 

def LLVBARS(S,N):         #求N周期内S最低值到当前周期数, 返回序列
    return pd.Series(S).rolling(N).apply(lambda x: np.argmin(x[::-1]),raw=True).values    
  
def MA(S,N):              #求序列的N日简单移动平均值，返回序列                    
    return pd.Series(S).rolling(N).mean().values  
  
def EMA(S,N):             #指数移动平均,为了精度 S>4*N  EMA至少需要120周期     alpha=2/(span+1)    
    return pd.Series(S).ewm(span=N, adjust=False).mean().values     

def SMA(S, N, M=1):       #中国式的SMA,至少需要120周期才精确 (雪球180周期)    alpha=1/(1+com)    
    return pd.Series(S).ewm(alpha=M/N,adjust=False).mean().values           #com=N-M/M

def WMA(S, N):            #通达信S序列的N日加权移动平均 Yn = (1*X1+2*X2+3*X3+...+n*Xn)/(1+2+3+...+Xn)
    return pd.Series(S).rolling(N).apply(lambda x:x[::-1].cumsum().sum()*2/N/(N+1),raw=True).values 

def DMA(S, A):            #求S的动态移动平均，A作平滑因子,必须 0<A<1  (此为核心函数，非指标）
    if isinstance(A,(int,float)):  return pd.Series(S).ewm(alpha=A,adjust=False).mean().values    
    A=np.array(A);   A[np.isnan(A)]=1.0;   Y= np.zeros(len(S));   Y[0]=S[0]     
    for i in range(1,len(S)): Y[i]=A[i]*S[i]+(1-A[i])*Y[i-1]      #A支持序列 by jqz1226         
    return Y             
  
def AVEDEV(S, N):         #平均绝对偏差  (序列与其平均值的绝对差的平均值)   
    return pd.Series(S).rolling(N).apply(lambda x: (np.abs(x - x.mean())).mean()).values 

def SLOPE(S, N):          #返S序列N周期回线性回归斜率            
    return pd.Series(S).rolling(N).apply(lambda x: np.polyfit(range(N),x,deg=1)[0],raw=True).values

def FORCAST(S, N):        #返回S序列N周期回线性回归后的预测值， jqz1226改进成序列出    
    return pd.Series(S).rolling(N).apply(lambda x:np.polyval(np.polyfit(range(N),x,deg=1),N-1),raw=True).values  

def LAST(S, A, B):        #从前A日到前B日一直满足S_BOOL条件, 要求A>B & A>0 & B>=0 
    return np.array(pd.Series(S).rolling(A+1).apply(lambda x:np.all(x[::-1][B:]),raw=True),dtype=bool)
  
#------------------   1级：应用层函数(通过0级核心函数实现）使用方法请参考通达信--------------------------------
def COUNT(S, N):                       # COUNT(CLOSE>O, N):  最近N天满足S_BOO的天数  True的天数
    return SUM(S,N)    

def EVERY(S, N):                       # EVERY(CLOSE>O, 5)   最近N天是否都是True
    return  IF(SUM(S,N)==N,True,False)                    
  
def EXIST(S, N):                       # EXIST(CLOSE>3010, N=5)  n日内是否存在一天大于3000点  
    return IF(SUM(S,N)>0,True,False)

def FILTER(S, N):                      # FILTER函数，S满足条件后，将其后N周期内的数据置为0, FILTER(C==H,5)
    for i in range(len(S)): S[i+1:i+1+N]=0  if S[i] else S[i+1:i+1+N]        
    return S                           # 例：FILTER(C==H,5) 涨停后，后5天不再发出信号 
  
def BARSLAST(S):                       #上一次条件成立到当前的周期, BARSLAST(C/REF(C,1)>=1.1) 上一次涨停到今天的天数 
    M=np.concatenate(([0],np.where(S,1,0)))  
    for i in range(1, len(M)):  M[i]=0 if M[i] else M[i-1]+1    
    return M[1:]                       

def BARSLASTCOUNT(S):                  # 统计连续满足S条件的周期数        by jqz1226
    rt = np.zeros(len(S)+1)            # BARSLASTCOUNT(CLOSE>OPEN)表示统计连续收阳的周期数
    for i in range(len(S)): rt[i+1]=rt[i]+1  if S[i] else rt[i+1]
    return rt[1:]  
  
def BARSSINCEN(S, N):                  # N周期内第一次S条件成立到现在的周期数,N为常量  by jqz1226
    return pd.Series(S).rolling(N).apply(lambda x:N-1-np.argmax(x) if np.argmax(x) or x[0] else 0,raw=True).fillna(0).values.astype(int)
  
def CROSS(S1, S2):                     # 判断向上金叉穿越 CROSS(MA(C,5),MA(C,10))  判断向下死叉穿越 CROSS(MA(C,10),MA(C,5))   
    return np.concatenate(([False], np.logical_not((S1>S2)[:-1]) & (S1>S2)[1:]))    # 不使用0级函数,移植方便  by jqz1226
    
def LONGCROSS(S1,S2,N):                # 两条线维持一定周期后交叉,S1在N周期内都小于S2,本周期从S1下方向上穿过S2时返回1,否则返回0         
    return  np.array(np.logical_and(LAST(S1<S2,N,1),(S1>S2)),dtype=bool)            # N=1时等同于CROSS(S1, S2)
    
def VALUEWHEN(S, X):                   # 当S条件成立时,取X的当前值,否则取VALUEWHEN的上个成立时的X值   by jqz1226
    return pd.Series(np.where(S,X,np.nan)).ffill().values  

def BETWEEN(S, A, B):                  # S处于A和B之间时为真。 包括 A<S<B 或 A>S>B
    return ((A<S) & (S<B)) | ((A>S) & (S>B))  

def TOPRANGE(S):                       # TOPRANGE(HIGH)表示当前最高价是近多少周期内最高价的最大值 by jqz1226
    rt = np.zeros(len(S))
    for i in range(1,len(S)):  rt[i] = np.argmin(np.flipud(S[:i]<S[i]))
    return rt.astype('int')

def LOWRANGE(S):                       # LOWRANGE(LOW)表示当前最低价是近多少周期内最低价的最小值 by jqz1226
    rt = np.zeros(len(S))
    for i in range(1,len(S)):  rt[i] = np.argmin(np.flipud(S[:i]>S[i]))
    return rt.astype('int')
  
  
#------------------   2级：技术指标函数(全部通过0级，1级函数实现） ------------------------------
def MACD(CLOSE,SHORT=12,LONG=26,M=9):             # EMA的关系，S取120日，和雪球小数点2位相同
    DIF = EMA(CLOSE,SHORT)-EMA(CLOSE,LONG);  # macd 
    DEA = EMA(DIF,M);     MACD=(DIF-DEA)*2
    return RD(EMA(CLOSE,SHORT)), RD(EMA(CLOSE,LONG)), RD(DIF),RD(DEA),RD(MACD)

def KDJ(CLOSE,HIGH,LOW, N=9,M1=3,M2=3):           # KDJ指标
    RSV = (CLOSE - LLV(LOW, N)) / (HHV(HIGH, N) - LLV(LOW, N)) * 100
    K = EMA(RSV, (M1*2-1));    D = EMA(K,(M2*2-1));        J=K*3-D*2
    return K, D, J

def RSI(CLOSE, N=24):                           # RSI指标,和通达信小数点2位相同
    DIF = CLOSE-REF(CLOSE,1) 
    return RD(SMA(MAX(DIF,0), N) / SMA(ABS(DIF), N) * 100)  

def WR(CLOSE, HIGH, LOW, N=10, N1=6):            #W&R 威廉指标
    WR = (HHV(HIGH, N) - CLOSE) / (HHV(HIGH, N) - LLV(LOW, N)) * 100
    WR1 = (HHV(HIGH, N1) - CLOSE) / (HHV(HIGH, N1) - LLV(LOW, N1)) * 100
    return RD(WR), RD(WR1)

def BIAS(CLOSE,L1=6, L2=12, L3=24):              # BIAS乖离率
    BIAS1 = (CLOSE - MA(CLOSE, L1)) / MA(CLOSE, L1) * 100
    BIAS2 = (CLOSE - MA(CLOSE, L2)) / MA(CLOSE, L2) * 100
    BIAS3 = (CLOSE - MA(CLOSE, L3)) / MA(CLOSE, L3) * 100
    return RD(BIAS1), RD(BIAS2), RD(BIAS3)

def BOLL(CLOSE,N=20, P=2):                       #BOLL指标，布林带    
    MID = MA(CLOSE, N); 
    UPPER = MID + STD(CLOSE, N) * P
    LOWER = MID - STD(CLOSE, N) * P
    return RD(UPPER), RD(MID), RD(LOWER)    

def PSY(CLOSE,N=12, M=6):  
    PSY=COUNT(CLOSE>REF(CLOSE,1),N)/N*100
    PSYMA=MA(PSY,M)
    return RD(PSY),RD(PSYMA)

def CCI(CLOSE,HIGH,LOW,N=14):  
    TP=(HIGH+LOW+CLOSE)/3
    return (TP-MA(TP,N))/(0.015*AVEDEV(TP,N))
        
def ATR(CLOSE,HIGH,LOW, N=20):                    #真实波动N日平均值
    TR = MAX(MAX((HIGH - LOW), ABS(REF(CLOSE, 1) - HIGH)), ABS(REF(CLOSE, 1) - LOW))
    return MA(TR, N)

def BBI(CLOSE,M1=3,M2=6,M3=12,M4=20):             #BBI多空指标   
    return (MA(CLOSE,M1)+MA(CLOSE,M2)+MA(CLOSE,M3)+MA(CLOSE,M4))/4    

def DMI(CLOSE,HIGH,LOW,M1=14,M2=6):               #动向指标：结果和同花顺，通达信完全一致
    TR = SUM(MAX(MAX(HIGH - LOW, ABS(HIGH - REF(CLOSE, 1))), ABS(LOW - REF(CLOSE, 1))), M1)
    HD = HIGH - REF(HIGH, 1);     LD = REF(LOW, 1) - LOW
    DMP = SUM(IF((HD > 0) & (HD > LD), HD, 0), M1)
    DMM = SUM(IF((LD > 0) & (LD > HD), LD, 0), M1)
    PDI = DMP * 100 / TR;         MDI = DMM * 100 / TR
    ADX = MA(ABS(MDI - PDI) / (PDI + MDI) * 100, M2)
    ADXR = (ADX + REF(ADX, M2)) / 2
    return PDI, MDI, ADX, ADXR  

def TAQ(HIGH,LOW,N):                               #唐安奇通道(海龟)交易指标，大道至简，能穿越牛熊
    UP=HHV(HIGH,N);    DOWN=LLV(LOW,N);    MID=(UP+DOWN)/2
    return UP,MID,DOWN

def KTN(CLOSE,HIGH,LOW,N=20,M=10):                 #肯特纳交易通道, N选20日，ATR选10日
    MID=EMA((HIGH+LOW+CLOSE)/3,N)
    ATRN=ATR(CLOSE,HIGH,LOW,M)
    UPPER=MID+2*ATRN;   LOWER=MID-2*ATRN
    return UPPER,MID,LOWER       
  
def TRIX(CLOSE,M1=12, M2=20):                      #三重指数平滑平均线
    TR = EMA(EMA(EMA(CLOSE, M1), M1), M1)
    TRIX = (TR - REF(TR, 1)) / REF(TR, 1) * 100
    TRMA = MA(TRIX, M2)
    return TRIX, TRMA

def VR(CLOSE,VOL,M1=26):                            #VR容量比率
    LC = REF(CLOSE, 1)
    return SUM(IF(CLOSE > LC, VOL, 0), M1) / SUM(IF(CLOSE <= LC, VOL, 0), M1) * 100

def EMV(HIGH,LOW,VOL,N=14,M=9):                     #简易波动指标 
    VOLUME=MA(VOL,N)/VOL;       MID=100*(HIGH+LOW-REF(HIGH+LOW,1))/(HIGH+LOW)
    EMV=MA(MID*VOLUME*(HIGH-LOW)/MA(HIGH-LOW,N),N);    MAEMV=MA(EMV,M)
    return EMV,MAEMV


def DPO(CLOSE,M1=20, M2=10, M3=6):                  #区间震荡线
    DPO = CLOSE - REF(MA(CLOSE, M1), M2);    MADPO = MA(DPO, M3)
    return DPO, MADPO

def BRAR(OPEN,CLOSE,HIGH,LOW,M1=26):                 #BRAR-ARBR 情绪指标  
    AR = SUM(HIGH - OPEN, M1) / SUM(OPEN - LOW, M1) * 100
    BR = SUM(MAX(0, HIGH - REF(CLOSE, 1)), M1) / SUM(MAX(0, REF(CLOSE, 1) - LOW), M1) * 100
    return AR, BR

def DFMA(CLOSE,N1=10,N2=50,M=10):                    #平行线差指标 
    DIF=MA(CLOSE,N1)-MA(CLOSE,N2); DIFMA=MA(DIF,M)   #通达信指标叫DMA 同花顺叫新DMA
    return DIF,DIFMA

def MTM(CLOSE,N=12,M=6):                             #动量指标
    MTM=CLOSE-REF(CLOSE,N);         MTMMA=MA(MTM,M)
    return MTM,MTMMA

def MASS(HIGH,LOW,N1=9,N2=25,M=6):                   #梅斯线
    MASS=SUM(MA(HIGH-LOW,N1)/MA(MA(HIGH-LOW,N1),N1),N2)
    MA_MASS=MA(MASS,M)
    return MASS,MA_MASS
  
def ROC(CLOSE,N=12,M=6):                             #变动率指标
    ROC=100*(CLOSE-REF(CLOSE,N))/REF(CLOSE,N);    MAROC=MA(ROC,M)
    return ROC,MAROC  

def EXPMA(CLOSE,N1=12,N2=50):                        #EMA指数平均数指标
    return EMA(CLOSE,N1),EMA(CLOSE,N2);

def OBV(CLOSE,VOL):                                  #能量潮指标
    return SUM(IF(CLOSE>REF(CLOSE,1),VOL,IF(CLOSE<REF(CLOSE,1),-VOL,0)),0)/10000

def MFI(CLOSE,HIGH,LOW,VOL,N=14):                    #MFI指标是成交量的RSI指标
    TYP = (HIGH + LOW + CLOSE)/3
    V1=SUM(IF(TYP>REF(TYP,1),TYP*VOL,0),N)/SUM(IF(TYP<REF(TYP,1),TYP*VOL,0),N)  
    return 100-(100/(1+V1))     
  
def ASI(OPEN,CLOSE,HIGH,LOW,M1=26,M2=10):            #振动升降指标
    LC=REF(CLOSE,1);      AA=ABS(HIGH-LC);     BB=ABS(LOW-LC);
    CC=ABS(HIGH-REF(LOW,1));   DD=ABS(LC-REF(OPEN,1));
    R=IF( (AA>BB) & (AA>CC),AA+BB/2+DD/4,IF( (BB>CC) & (BB>AA),BB+AA/2+DD/4,CC+DD/4));
    X=(CLOSE-LC+(CLOSE-OPEN)/2+LC-REF(OPEN,1));
    SI=16*X/R*MAX(AA,BB);   ASI=SUM(SI,M1);   ASIT=MA(ASI,M2);
    return ASI,ASIT   

def XSII(CLOSE, HIGH, LOW, N=102, M=7):              #薛斯通道II  
    AA  = MA((2*CLOSE + HIGH + LOW)/4, 5)            #最新版DMA才支持 2021-12-4
    TD1 = AA*N/100;   TD2 = AA*(200-N) / 100
    CC =  ABS((2*CLOSE + HIGH + LOW)/4 - MA(CLOSE,20))/MA(CLOSE,20)
    DD =  DMA(CLOSE,CC);    TD3=(1+M/100)*DD;      TD4=(1-M/100)*DD
    return TD1, TD2, TD3, TD4  
  
  
  #望大家能提交更多指标和函数  https://github.com/mpquant/MyTT


from setuptools import setup, find_packages


with open("project-description.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup_args = dict(
    name='TradingBot',
    version='0.1.0',
    description='A Python toolkit for strategy design, backtesting, and algorithm trading',
    long_description_content_type="text/markdown",
    long_description=long_description,
    license='MIT',
    packages=find_packages(),
    author='Dayu Yang',
    author_email='dayu@udel.edu',
    keywords=['Stock', 'Trading'],
    url='https://github.com/dayuyang1999/TradingBot', 
    download_url='https://pypi.org/project/TradingBot'
)


install_requires = [
    'alpaca-trade-api>=2.0.0,<3.0.0',
    'jupyter'
    ]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from datetime import datetime
#from tradebot1 import *
#import backtrader as bt
import warnings
warnings.filterwarnings("ignore")

import yfinance as yf
from .tradebot import EMA, RD, MA


def get_data(tiker='^IXIC', start="2022-05-05", end="2022-05-06", which='Close', interval='1m'):
    '''
    get_data from yahoo finance
    
    # Valid periods: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max
    # Valid intervals: 1m,2m,5m,15m,30m,60m,90m,1h,1d,5d,1wk,1mo,3mo
    # see https://github.com/ranaroussi/yfinance/blob/9eef951acc70121e65825ad25e7afd2edd4c3e4b/yfinance/multi.py
    '''
    def reformulate_date(df):
        df.columns = df.columns.map(''.join)
        df = df.rename_axis('Date').reset_index()
        return df
    data = yf.download(tickers = tiker, start=start, end=end, interval = interval, progress=False)
    data['pct_change'] = data[which].pct_change()
    data['log_ret'] = np.log(data[which].shift(-1)) - np.log(data[which])
    data = reformulate_date(data)
    #print("most recent date is ", data.iloc[-1, 0])
    return data


from simplegmail import Gmail








if __name__ == '__main__':
    gmail = Gmail() # will open a browser window to ask you to log in and authenticate

    params = {
    "to": "dayu@udel.com",
    "sender": "yangdayu1997@gmail.com",
    "subject": 'Stock Bot Stop working! Need attention immediately!',
    "msg_html": "<h1>check https://app.alpaca.markets/paper/dashboard/overview</h1><br /> Blank for later usage.",
    "msg_plain": "Hi\nThis is a plain text email.",
    "signature": True  # use my account signature
    }
    message = gmail.send_message(**params) 